import Vue from 'vue';
import * as axios from 'axios';
import commonConfig from '@/assets/config';
import store from '@/store';
import { Toast } from 'vant';
import * as TPTJS from 'tpt-js-sdk';

Vue.use(Toast);
Toast.allowMultiple(true);

/**
 * @author john.bian
 * @description axios封装
 * @
 */
class Fetch {
  public cancelToken = axios.default.CancelToken;
  public source = this.cancelToken.source();

  constructor(public ax: axios.AxiosInstance) { }

  /**
   * get 请求
   * @param url 请求地址
   */
  public async get(url: string, baseConfig?: any): Promise<any> {
    const config = {
      cancelToken: this.source.token,
      shouldHandleError: baseConfig ? baseConfig.shouldHandleError : true
    };
    try {
      const ret = await this.ax.get(encodeURI(url), config);
      return this.handleResponse(url, ret);
    } catch (err) {
      return this.commonErrorHandle(err);
    }
  }

  /**
   * post 请求
   * @param url 请求地址
   * @param params 请求参数
   */
  public async post(url: any, params: any, baseConfig?: any): Promise<any> {
    const config = {
      cancelToken: this.source.token,
      shouldHandleError: baseConfig ? baseConfig.shouldHandleError : true
    };
    try {
      const ret = await this.ax.post(encodeURI(url), params, config);
      return this.handleResponse(url, ret);
    } catch (err) {
      return this.commonErrorHandle(err);
    }
  }

  /**
   * 用户取消请求 比如返回
   */
  public cancel(): void {
    this.source.cancel('bxsccancel');
  }

  private handleResponse(url: string, ret: axios.AxiosResponse): any {
    // message.destroy();
    // if (ret.data.code !== '0000') {
    //   // message.error(ret.data.message);
    //   return ret.data;
    // }
    return ret.data;
  }

  private commonErrorHandle(err: any): any {
    if (err.response && err.response.data) {
      return err.response.data;
    }
    return err;
  }
}

const instance = axios.default.create({
  baseURL: commonConfig.baseUrl,
  timeout: 60 * 1000
});

instance.interceptors.request.use((config): any => {
  config.headers = {
    'x-ac-mc-type': 'gateway.user',
    'x-ac-channel-id': 'KHT',
    'x-ac-token-ticket': (store.state as any).user.userInfo.token
  };
  config.headers['Content-Type'] = 'application/json';
  store.dispatch('num/add');
  return config;
});

instance.interceptors.response.use(
  (response): any => {
    const { shouldHandleError }: any = response.config;
    store.dispatch('num/del');
    if (response.data.code === '0000') {
      return response;
    } else if (
      commonConfig.responseCode.businessError.indexOf(response.data.code) !== -1
    ) {
      TPTJS.khtAppRouteRequest(0, '', 1, 'native', 'LogIn');
      return response;
    } else if (response.data.code[0] === '9' && shouldHandleError) {
      Toast(commonConfig.responseCode.systemError);
      return response;
    } else if (response.data.code === '8999') {
      TPTJS.khtAppRouteRequest(0, '', 1, 'native', 'RealNameAuthentication');
    } else if (
      Number(response.data.code[0]) >= 0 &&
      Number(response.data.code[0]) <= 8 && shouldHandleError
    ) {
      Toast(response.data.desc);
      return response;
    } else {
      return response;
    }
  },
  (err): any => {
    store.dispatch('num/del');
    if (err.message === 'bxsccancel') {
      return Promise.resolve(err.response);
    } else if (err.toString().indexOf('Network') !== -1) {
      Toast(commonConfig.responseCode.networkError);
      return err;
    } else {
      Toast(commonConfig.responseCode.systemError);
      return err.response;
    }
  }
);

const fetch = new Fetch(instance);

export default fetch;
